package com.example.nto.service.impl;

import com.example.nto.dto.UsersDTO;
import com.example.nto.entity.*;
import com.example.nto.exception.PersonNotFoundException;
import com.example.nto.repository.*;
import com.example.nto.service.UsersService;
import com.example.nto.util.UsersMapper;
import lombok.RequiredArgsConstructor;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

@Service
@RequiredArgsConstructor
public class UsersServiceImpl implements UsersService {

    private final UsersRepository usersRepository;
    private final CredentialsRepository credentialsRepository;
    private final AuthorityRepository rolesRepository;
    private final PasswordEncoder passwordEncoder;

    @Override
    public List<UsersDTO> getAllUsers() {
        return usersRepository.findAll().stream()
                .map(UsersMapper::convertDTO)
                .collect(Collectors.toList());
    }




    @Override
    public UsersDTO getUserbyId(Long id) {
        return usersRepository.findById(id)
                .map(UsersMapper::convertDTO)
                .orElseThrow(() -> new PersonNotFoundException("Person not found!"));
    }


    @Override
    public Users updateEmployee(long id, Users newUser) {
        Optional<Users> optionalEmployee = usersRepository.findById(id);
        if (optionalEmployee.isEmpty()) throw new RuntimeException("No such user with id " + id);

        Pass pass = new Pass();
        Users users = optionalEmployee.get();
        users.setName(newUser.getName());
        users.setUsername(newUser.getUsername());
        users.setPhoto(newUser.getPhoto());
        users.setPosition(newUser.getPosition());
        users.setLastVisit(newUser.getLastVisit());

        return usersRepository.save(users);
    }

    @Override
    public String blockUser(String username) {
        Optional<Users> users = usersRepository.findByUsername(username);

        if (users.isPresent()) {
            Users user = users.get();
            user.setIsCardBlocked(true);
            usersRepository.save(user);
            return "Blocked user";
        }
        throw new RuntimeException("No such user with username " + username);
    }
    @Override
    public String unblockUser(String username) {
        Optional<Users> users = usersRepository.findByUsername(username);

        if (users.isPresent()) {
            Users user = users.get();
            user.setIsCardBlocked(false);
            usersRepository.save(user);
            return "Unblocked user";
        }
        throw new PersonNotFoundException("No such user with username " + username);
    }


    @Override
    public UsersDTO getUserByUsername(String username) {
        Optional<Users> optionalUsers = usersRepository.findByUsername(username);

        if (optionalUsers.isEmpty()) {
            throw new PersonNotFoundException("User with username " + username + "not found");
        }
        return UsersMapper.convertDTO(optionalUsers.get());
    }
}
